package de.laliluna.howto.application;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.faces.FacesException;
import javax.faces.application.ViewExpiredException;
import javax.faces.application.NavigationHandler;
import javax.faces.application.ProjectStage;
import javax.faces.component.UICommand;
import javax.faces.context.ExceptionHandler;
import javax.faces.context.ExceptionHandlerWrapper;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ExceptionQueuedEvent;
import javax.faces.event.ExceptionQueuedEventContext;
import javax.faces.event.PhaseId;
import java.util.Iterator;
import java.util.Map;

/**
 * Exception handler, which fixes a bug in the current JSF 2 snapshot (AbortProcessingException)
 * and which sends the user to a viewexpired page if the view is expired and the application
 * is in production mode.
 * You need to switch the state saving in web.xml to server in order to test the latter.
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 22.10.2009
 */
public class CustomExceptionHandler extends ExceptionHandlerWrapper {
	final Logger logger = LoggerFactory.getLogger(CustomExceptionHandler.class);
	private ExceptionHandler wrapped;

	public CustomExceptionHandler(ExceptionHandler wrapped) {
		this.wrapped = wrapped;
	}

	public ExceptionHandler getWrapped() {
		return wrapped;
	}

	@Override
	public void handle() throws FacesException {
		Iterator<ExceptionQueuedEvent> iterator = getUnhandledExceptionQueuedEvents().iterator();
		while (iterator.hasNext()) {
			ExceptionQueuedEvent event = iterator.next();
			ExceptionQueuedEventContext context = (ExceptionQueuedEventContext) event.getSource();

			// Workaround as current JSF 2 Snapshot swallows the exception of action methods
			if (context.getException() instanceof AbortProcessingException) {
				if (context.getPhaseId() == PhaseId.INVOKE_APPLICATION && context.getComponent() instanceof UICommand &&
						!(context.getException().getCause() instanceof AbortProcessingException)) {
					iterator.remove();
					throw new FacesException(context.getException().getCause().getMessage(), context.getException().getCause());
				}

			} else if (context.getException() instanceof ViewExpiredException) {
				ViewExpiredException exception = (ViewExpiredException) context.getException();
				FacesContext fc = FacesContext.getCurrentInstance();
				Map<String, Object> requestMap = fc.getExternalContext().getRequestMap();
				NavigationHandler nav =
						fc.getApplication().getNavigationHandler();
				try {
					// in development mode, we want to show the normal debug screen of facelets
					if(fc.isProjectStage(ProjectStage.Development)){
						fc.getExternalContext().getRequestMap().put("com.sun.faces.error.view", fc.getViewRoot());
						throw new FacesException(context.getException().getMessage(), context.getException());
						 
					} else{
					// Push some useful stuff to the request scope for
					// use in the page
					requestMap.put("currentViewId", exception.getViewId());

					nav.handleNavigation(fc, null, "viewexpired");
					fc.renderResponse();

					}
				} finally {
					iterator.remove();
				}
			}


		}
		super.handle();
	}
}
