package de.laliluna.howto.application;

import com.sun.faces.application.view.MultiViewHandler;
import com.sun.faces.util.FacesLogger;
import com.sun.faces.util.Util;

import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.faces.context.FacesContext;
import javax.faces.context.ExternalContext;
import javax.faces.application.ViewHandler;

public class FriendlyUrlViewHandler extends MultiViewHandler {

	private static final Logger logger = FacesLogger.APPLICATION.getLogger();
	private UrlAnalyser analyser = new UrlAnalyser();

	@Override
	protected String derivePhysicalViewId(FacesContext ctx, String rawViewId, boolean checkPhysical) {
		if (rawViewId != null) {
			String mapping = Util.getFacesMapping(ctx);
			String viewId;
			if (mapping != null) {


				if (!Util.isPrefixMapped(mapping)) {
					viewId = convertViewId(ctx, rawViewId);
					return checkExistence(ctx, viewId);
				} else {
					// The following code determines possible view ids from a URL. /books/foo could point to to /books/foo.xhtml
					// or /books.xhtml with parameter foo
					List<String> possibleViewIds = analyser.analyse(rawViewId);
					if (possibleViewIds == null) {
						possibleViewIds = Arrays.asList(rawViewId);
					}
					for (String path : possibleViewIds) {

						viewId = normalizeRequestURI(path, mapping);
						if (viewId.equals(mapping)) {
							// The request was to the FacesServlet only - no
							// path info
							// on some containers this causes a recursion in the
							// RequestDispatcher and the request appears to hang.
							// If this is detected, return status 404
							send404Error(ctx);
						}

						viewId = checkExistence(ctx, viewId);
						if (viewId != null) {
							return viewId;
						}
					}
				}


			}

		}
		return rawViewId;
	}

	protected String checkExistence(FacesContext ctx, String viewId) {
		try {
			return ((ctx.getExternalContext().getResource(viewId) != null) ? viewId : null);

		} catch (MalformedURLException mue) {
			if (logger.isLoggable(Level.SEVERE)) {
				logger.log(Level.SEVERE,
						mue.toString(),
						mue);
			}
			return null;
		}
	}

	@Override
	protected void addViewParameters(FacesContext ctx, String viewId, Map<String, List<String>> existingParameters) {
		super.addViewParameters(ctx, viewId, existingParameters);
	}

	@Override
	public String getActionURL(FacesContext context, String viewId) {
		
		return super.getActionURL(context, viewId);
	}

	@Override
	public String getBookmarkableURL(FacesContext context, String viewId, Map<String, List<String>> parameters,
																	 boolean includeViewParams) {
		Map<String, List<String>> params;
		if (includeViewParams) {
			params = getFullParameterList(context, viewId, parameters);
		} else {
			params = parameters;
		}
		// if there is not more then one param add it to the URL else build URL as normal
		ExternalContext ectx = context.getExternalContext();
		ViewHandler viewHandler = Util.getViewHandler(context);
		String actionUrl = viewHandler.getActionURL(context, viewId);
		if (params.size() > 1 || !actionUrl.endsWith(".xhtml")) {
			return ectx.encodeActionURL(
					ectx.encodeBookmarkableURL(actionUrl, params));
		} else {
			// strip .xhtml
			StringBuilder builder = new StringBuilder(actionUrl.substring(0, actionUrl.length() - 6));
			if (params.size() == 1)
				builder.append('/').append(params.values().iterator().next().get(0));
			return ectx.encodeActionURL(builder.toString());

		}

	}


}
