package de.laliluna.howto.application;

import de.laliluna.howto.services.PicoObjectProvider;
import org.apache.log4j.Logger;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.el.PropertyNotFoundException;
import javax.el.PropertyNotWritableException;
import java.beans.FeatureDescriptor;
import java.util.Iterator;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 14.10.2009
 */
public class PicoBeanELResolver extends ELResolver {
	private static Logger logger = Logger.getLogger(PicoBeanELResolver.class);

	public PicoBeanELResolver() {
		// init the Picocontainer
//		It is better to add a startup listener instead of doing this here !!!!
		PicoObjectProvider.getInstance().startUp();
	}

	/**
	 * Is called to resolve an expression. 
	 * @param elContext
	 * @param base
	 * @param property
	 * @return
	 */
	public Object getValue(ELContext elContext, Object base, Object property) {
		// the resolver resolves only names without prefix.
		if (base != null) {
			return null;
		}
		if (property == null) {
			throw new PropertyNotFoundException("Could not resolve property, as property name is null");
		}

		Object result = null;
		PicoObjectProvider provider = PicoObjectProvider.getInstance();
		result = provider.provide(property.toString());

		if (result != null) {
			elContext.setPropertyResolved(true);
			if (logger.isDebugEnabled())
				logger.debug("Got value for base (" + base + ") and property (" + property + ")");
		}

		return result;
	}

	public Class<?> getType(ELContext elContext, Object base, Object property) {
		Object value = getValue(elContext, base, property);

		return value != null ? value.getClass() : null;
	}

	public void setValue(ELContext elContext, Object base, Object property, Object val) {
		Object value = getValue(elContext, base, property);
		if (value != null) { // should never happen, as readonly returns true
			throw new PropertyNotWritableException("Objects resolved by the Pico Container are not writable");
		}
	}

	public boolean isReadOnly(ELContext elContext, Object base, Object property) {
		if (logger.isDebugEnabled())
			logger.debug("is read only test for base (" + base + ") and property (" + property + ")");

		return getValue(elContext, base, property) != null;
	}

	// the following methods are for IDE support


	public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext elContext, Object o) {
		return null;
	}

	public Class<?> getCommonPropertyType(ELContext elContext, Object o) {
		return null;
	}
}
