package de.laliluna.howto.application;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ApplicationScoped;
import javax.servlet.http.HttpSession;
import java.util.concurrent.atomic.AtomicLong;
import java.util.HashMap;
import java.util.Map;
import java.util.Collection;
import java.util.ArrayList;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 23.10.2009
 */
@ManagedBean @ApplicationScoped
public class WizardHelper {

	public static final String WIZARD_SCOPE_KEY = "de.laliluna.WizardScope";

	private AtomicLong wizardId = new AtomicLong();

	public void start() {
		Collection collection = new ArrayList();
		int counter = 0;
		for (Object item : collection) {
			// render content
			counter++;
		}
		CustomFacesContext context = CustomFacesContext.getCurrentInstance();
		context.setWizardId(String.valueOf(wizardId.incrementAndGet()));
	}

	public void end() {
		CustomFacesContext context = CustomFacesContext.getCurrentInstance();
		WizardScopeResolver.destroyScope(context);
		context.setWizardId(null);
	}

	public <T> T get(String key) {
		return (T) getOrCreateScope().get(key);
	}

	public void put(String key, Object value) {
		getOrCreateScope().put(key, value);
	}

	private WizardScopeResolver.WizardScope getOrCreateScope() {
		CustomFacesContext context = CustomFacesContext.getCurrentInstance();
		if (!context.isWizardEnabled())
					throw new IllegalStateException("Wizard scope has not been started");

						HttpSession session = (HttpSession) context.getExternalContext().getSession(true);
		WizardScopeResolver.WizardScope wizardScope = null;

		synchronized (session.getId()) {
			Map<String, WizardScopeResolver.WizardScope> scopes = (Map<String, WizardScopeResolver.WizardScope>) session.getAttribute(
					WIZARD_SCOPE_KEY);
			if (scopes == null) {
				scopes = new HashMap<String, WizardScopeResolver.WizardScope>();
				wizardScope = new WizardScopeResolver.WizardScope();
				scopes.put(context.getWizardId(), wizardScope);
				session.setAttribute(WIZARD_SCOPE_KEY, scopes);
			} else {
				wizardScope = scopes.get(context.getWizardId());

				if(wizardScope == null){
					wizardScope = new WizardScopeResolver.WizardScope();
					scopes.put(context.getWizardId(), wizardScope);
				}
			}
		}

		return wizardScope;
	}

}
