package de.laliluna.howto.application;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.el.ELResolver;
import javax.el.ELContext;
import javax.el.PropertyNotFoundException;
import javax.servlet.http.HttpSession;
import java.util.Iterator;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;
import java.beans.FeatureDescriptor;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 22.10.2009
 */
public class WizardScopeResolver extends ELResolver {

	private static final String SCOPE_NAME = "wizardScope";
	final static Logger logger = LoggerFactory.getLogger(WizardScopeResolver.class);

	private WizardScope getScope(ELContext elContext) {
		CustomFacesContext ctx = CustomFacesContext.getCurrentInstance();

		HttpSession session = (HttpSession) ctx.getExternalContext().getSession(true);
		WizardScopeResolver.WizardScope wizardScope = null;
		synchronized (session.getId()) {
			Map<String, WizardScope> scopes = (Map<String, WizardScopeResolver.WizardScope>) session.getAttribute(
					WizardHelper.WIZARD_SCOPE_KEY);
			if (scopes == null) {
				scopes = new HashMap<String, WizardScope>();
				wizardScope = new WizardScopeResolver.WizardScope();
				scopes.put(ctx.getWizardId(), wizardScope);
				logger.debug("Created wizard scope with id ({})", ctx.getWizardId());
				session.setAttribute(WizardHelper.WIZARD_SCOPE_KEY, scopes);
			} else {
				wizardScope = scopes.get(ctx.getWizardId());

				if (wizardScope == null) {
					wizardScope = new WizardScopeResolver.WizardScope();
					scopes.put(ctx.getWizardId(), wizardScope);
					logger.debug("Created wizard scope with id ({})", ctx.getWizardId());
				}
			}
		}

		return wizardScope;
	}

	public Object getValue(ELContext elContext, Object base, Object property) {
		if (property == null) {
			throw new PropertyNotFoundException("Could not resolve property, as property name is null");
		}
		if (base == null) {
			if (SCOPE_NAME.equals(property.toString())) {
				elContext.setPropertyResolved(true);
				return getScope(elContext);
			} else {
				return lookup(elContext, getScope(elContext), property.toString());
			}

		} else if (base instanceof WizardScope) {
			return lookup(elContext, (WizardScope) base, property.toString());
		}
		return null;
	}

	private Object lookup(ELContext elContext, WizardScope scope, String key) {
		Object result = scope.get(key);
		if (result != null)
			elContext.setPropertyResolved(true);
		return result;
	}


	public Class<?> getType(ELContext elContext, Object base, Object o1) {
		return Object.class;
	}

	public void setValue(ELContext elContext, Object base, Object o1, Object o2) {
		logger.debug("Setting value");
	}

	public boolean isReadOnly(ELContext elContext, Object base, Object o1) {
		return true;
	}

	public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext elContext, Object o) {
		return Collections.<FeatureDescriptor>emptyList().iterator();
	}

	public Class<?> getCommonPropertyType(ELContext elContext, Object base) {
		if (base != null) {
			return null;
		}
		return String.class;
	}

	public static void destroyScope(CustomFacesContext context) {
		HttpSession session = (HttpSession) context.getExternalContext().getSession(true);
		WizardScopeResolver.WizardScope wizardScope = null;
		synchronized (session.getId()) {
			Map<String, WizardScope> scopes = (Map<String, WizardScopeResolver.WizardScope>) session.getAttribute(
					WizardHelper.WIZARD_SCOPE_KEY);
			if (scopes != null) {
				wizardScope = scopes.remove(context.getWizardId());
				if (scopes.size() == 0)
					session.removeAttribute(WizardHelper.WIZARD_SCOPE_KEY);
			}
		}
		logger.debug("Destroyed wizard scope for id ({})",context.getWizardId());
	}


	public static class WizardScope extends ConcurrentHashMap<String, Object> {

	}
}
