package de.laliluna.howto.beans;

import de.laliluna.howto.model.Article;
import de.laliluna.howto.services.ServiceFactoryImp;
import de.laliluna.howto.services.ArticleService;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.bean.ViewScoped;
import java.util.*;
import java.io.Serializable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 20.10.2009
 */
@ManagedBean  @SessionScoped
public class AjaxTable implements Serializable{

	final Logger logger = LoggerFactory.getLogger(AjaxTable.class);
	private String sortCriteria;

	private List<Article> articles = ServiceFactoryImp.getInstance().getService(ArticleService.class).findAll();

	private List<Article> editArticles = new ArrayList<Article>();

	public AjaxTable() {
		editArticles =  ServiceFactoryImp.getInstance().getService(ArticleService.class).findAll();
	}

	public List<Article> getArticles() {
		return articles;
	}

	public void sort() {
		if (sortCriteria == null)
			return;
		logger.info("Sorting by {}", sortCriteria);
		if ("id".equals(sortCriteria))
			Collections.sort(articles, new Comparator<Article>() {
				public int compare(Article o1, Article o2) {
					return o1.getId().compareTo(o2.getId());
				}
			});
		else if ("name".equals(sortCriteria))
			Collections.sort(articles, new Comparator<Article>() {
				public int compare(Article o1, Article o2) {
					return o1.getName().compareTo(o2.getName());
				}
			});
		else if ("description".equals(sortCriteria))
			Collections.sort(articles, new Comparator<Article>() {
				public int compare(Article o1, Article o2) {
					return o1.getDescription().compareTo(o2.getDescription());
				}
			});
	}


	public String save() {
		ArticleService service = ServiceFactoryImp.getInstance().getService(ArticleService.class);
		for (Article article : editArticles) {
			logger.debug("Create or update {}", article);
			service.update(article);
		}
		return "ajaxTable?faces-redirect=true"; // does only work with @SessionScope not with @ViewScope
	}

	public List<Article> getEditArticles() {
		return editArticles;
	}

	public void setEditArticles(List<Article> editArticles) {
		this.editArticles = editArticles;
	}

	/**
	 * Method could be used if EL expressions like #{ajaxTable.sort('id')} are supported
	 *
	 * @param criteria
	 */
	public void sort(String criteria) {
		this.sortCriteria = criteria;
		sort();
	}


	public String getSortCriteria() {
		return sortCriteria;
	}

	public void setSortCriteria(String sortCriteria) {
		this.sortCriteria = sortCriteria;
	}
}
