package de.laliluna.howto.beans;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.CustomScoped;
import javax.faces.bean.ManagedProperty;
import javax.faces.context.FacesContext;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.validation.constraints.Min;

import de.laliluna.howto.application.WizardHelper;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 23.10.2009
 */
@ManagedBean
@CustomScoped("#{wizardScope}")
public class WizardBean {
	final Logger logger = LoggerFactory.getLogger(WizardBean.class);

	@ManagedProperty(value = "#{wizardHelper}")
	private WizardHelper wizardHelper;

	@NotNull(groups = Step1.class)
	@Size(min = 3, groups = Step1.class)
	private String name;

	@Min(value = 3, groups = Step2.class)
	private int age;

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getAge() {
		return age;
	}

	public void setAge(int age) {
		this.age = age;
	}

	public String start() {
		logger.debug("Starting wizard");
		wizardHelper.start();
		return "wizardstep1" + Globals.REDIRECT;
	}

	public String next() {
		logger.debug("Step 1");
		return "wizardstep2" + Globals.REDIRECT;
	}

	public String finish() {
		logger.debug("Finish");
		FacesContext.getCurrentInstance().getExternalContext().getFlash().put("name", name);
		FacesContext.getCurrentInstance().getExternalContext().getFlash().put("age", age);
		wizardHelper.end();
		return "wizardend" + Globals.REDIRECT;
	}


	// interfaces are just used as marker for the group validation
	public static interface Step1 {
	}

	public static interface Step2 {
	}

	public void setWizardHelper(WizardHelper wizardHelper) {
		this.wizardHelper = wizardHelper;
	}
}
