package de.laliluna.howto.components;

import com.sun.faces.util.FacesLogger;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.faces.component.UIViewParameter;
import javax.faces.context.FacesContext;

public class UIUrlParam extends UIViewParameter {

	private static final Logger logger = FacesLogger.APPLICATION.getLogger();
	private String from;
	private Pattern pattern;
	private String rawValue;
	/**
	 * <p>The standard component type for this component.</p>
	 */
	public static final String COMPONENT_TYPE = "de.laliluna.howto.components.UrlParameter";
	/**
	 * <p>The standard component family for this component.</p>
	 */
//	public static final String COMPONENT_FAMILY = "de.laliluna.howto.components.UrlParameter";

	public UIUrlParam() {
		super(); // will set the renderer to null, as this component is not rendered
	}


	public String getFrom() {
		return from;
	}

	/**
	 * Defines how the param is extracted from the URL. sample /books/show/$$
	 * In this sample the value fo $$ will be used as param value.
	 * The from value is transformed to a regular expression, where / is replaced
	 * with \/ and $$ with a group like ([^/]?)
	 * /books/show/$$ results in Pattern.compile("\\/books\\/show\\/([^/]?)
	 * @param from -
	 */
	public void setFrom(String from) {
		this.from = from;
		String regex = from.replace("/", "\\/").replaceFirst("\\$\\$", "([^/]?)");
		pattern = Pattern.compile(regex);
	}

	/**
	 * Extracts the param from the URL
	 * @param context
	 */
	@Override
	public void decode(FacesContext context) {
		if (context == null) {
			throw new NullPointerException();
		}

		String path = context.getExternalContext().getRequestPathInfo();
		String viewId = context.getViewRoot().getViewId();
		if (viewId.equals(path)) {
		} else {
			Matcher matcher = pattern.matcher(path);
			if (matcher.find() && matcher.groupCount() > 0) {
				String param = matcher.group(1);
				if (param != null) {
					setSubmittedValue(param);
				}
			}
		}
		rawValue = (String) getSubmittedValue();
		setValid(true);
	}

	/**
	 * This is only overwritten, because it uses a private field rawValue which
	 * we couldn't set from the decode method
	 * @param context
	 * @return
	 */
	@Override
	public String getStringValue(FacesContext context) {
		String result = null;
		if (hasValueExpression()) {
			result = getStringValueFromModel(context);
		} else {
			result = (null != rawValue) ? rawValue : (String) getValue();
		}
		return result;
	}

	/**
	 * This is only here as companion method to getStringValue
	 * @return
	 */
	private boolean hasValueExpression() {
		return null != getValueExpression("value");
	}

	
}
