package de.laliluna.howto.model;

import de.laliluna.howto.components.CompanyEmail;
import de.laliluna.howto.beans.InputBeanEmailValidation;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.Date;

public class Address extends AbstractModel{
	private Integer id;

	@NotNull @Size(min = 3)
	private String name;

	@NotNull @Size(min = 2, max = 10)
	private String street;
	private String city;
	private String country;
	private Salutation salutation;

	private Date moved;

	@CompanyEmail(groups = {InputBeanEmailValidation.class})
	private String email;

	private Phone phone;

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Phone getPhone() {
		return phone;
	}

	public void setPhone(Phone phone) {
		this.phone = phone;
	}

	public Address() {
	}

	public Address(String name, String city, String country, String street, Salutation salutation, Phone phone) {
		this.name = name;
		this.street = street;
		this.city = city;
		this.country = country;
		this.salutation = salutation;
		this.phone = phone;
	}

	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public Salutation getSalutation() {
		return salutation;
	}

	public void setSalutation(Salutation salutation) {
		this.salutation = salutation;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getStreet() {
		return street;
	}

	public void setStreet(String street) {
		this.street = street;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public Date getMoved() {
		return moved;
	}

	public void setMoved(Date moved) {
		this.moved = moved;
	}

	public Object clone(){
		Address address = new Address(name, city, country, street, salutation, null);
		address.setId(id);
		address.setEmail(email);
		address.setMoved(moved != null ? (Date) moved.clone() : null);
		address.setPhone(phone == null ? null : new Phone(phone.getCountryCode(), phone.getAreaPrefix(), phone.getCode()));
		return address;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("Address");
		sb.append("{id=").append(id);
		sb.append(", name='").append(name).append('\'');
		sb.append(", street='").append(street).append('\'');
		sb.append(", city='").append(city).append('\'');
		sb.append(", country='").append(country).append('\'');
		sb.append(", salutation=").append(salutation);
		sb.append(", moved=").append(moved);
		sb.append(", email='").append(email).append('\'');
		sb.append(", phone=").append(phone);
		sb.append('}');
		return sb.toString();
	}
}
