package de.laliluna.howto.services;

import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.text.MessageFormat;

/**
 * Redirects java.util.logging messages to log4j
 * Idea is taken from http://wiki.apache.org/myfaces/Trinidad_and_Common_Logging
 * <p/>
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 30.09.2009
 */
public class LogRedirect {

	static JDKLogHandler activeHandler;

	static Handler[] oldJavaHandlers;

	/**
	 * Activates this feature.
	 */
	public static void activate() {
		Logger rootLogger = LogManager.getLogManager().getLogger("");
		try {

			// remove old handlers
			oldJavaHandlers = rootLogger.getHandlers();
			for (Handler handler : rootLogger.getHandlers()) {
				rootLogger.removeHandler(handler);

			}
			// add our own
			activeHandler = new JDKLogHandler();
			activeHandler.setLevel(Level.ALL);
			rootLogger.addHandler(activeHandler);
			rootLogger.setLevel(Level.ALL);
			// done, let's check it right away!!!

			Logger.getLogger(LogRedirect.class.getName()).info("activated: sending JDK log messages to log4j Logging");
		} catch (Exception exc) {
			rootLogger.log(Level.SEVERE, "activation failed", exc);
		}
	}

	public static void deactivate() {
		Logger.getLogger(LogRedirect.class.getName()).info("deactivating redirect");

		Logger rootLogger = LogManager.getLogManager().getLogger("");
		rootLogger.removeHandler(activeHandler);
		for (Handler handler : oldJavaHandlers) {
			rootLogger.addHandler(handler);
		}
		oldJavaHandlers = null;
		Logger.getLogger(LogRedirect.class.getName()).info("deactivated");
	}

	protected static class JDKLogHandler extends Handler {
		private Map<String, org.apache.log4j.Logger> cachedLogs = new ConcurrentHashMap<String, org.apache.log4j.Logger>();

		private org.apache.log4j.Logger getLog(String logName) {
			org.apache.log4j.Logger log = cachedLogs.get(logName);
			if (log == null) {
				log = org.apache.log4j.Logger.getLogger(logName);
				cachedLogs.put(logName, log);
			}
			return log;
		}

		@Override
		public void publish(LogRecord record) {
			org.apache.log4j.Logger log = getLog(record.getLoggerName());
			String message = record.getMessage();
			Throwable exception = record.getThrown();
			Level level = record.getLevel();
			org.apache.log4j.Level log4jLevel = org.apache.log4j.Level.TRACE;

			if (level == Level.SEVERE) {
				log4jLevel = org.apache.log4j.Level.FATAL;
			} else if (level == Level.WARNING) {
				log4jLevel = org.apache.log4j.Level.WARN;
			} else if (level == Level.INFO) {
				log4jLevel = org.apache.log4j.Level.INFO;
			} else if (level == Level.CONFIG || level == Level.FINE || level == Level.FINER) {
				log4jLevel = org.apache.log4j.Level.DEBUG;
			}
			if(record.getParameters() != null)
				log.log(log4jLevel, MessageFormat.format(message, record.getParameters()),exception);
			else
				log.log(log4jLevel, message,exception);

		}

		@Override
		public void flush() {
			// nothing to do
		}

		@Override
		public void close() {
			// nothing to do
		}
	}

}
