package de.laliluna.howto.services;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 19.10.2009
 */
public class MessageFactory {

	/**
	 * Simple implementation. A real world implementation should provide fall back to default locate and
	 * support the summary + detailed message approach as the class {@link javax.faces.convert.MessageFactory}
	 *
	 * @param context
	 * @param messageId
	 * @param params
	 * @return
	 */
	public static FacesMessage getMessage(FacesContext context,
																 String messageId,
																 Object... params) {

		if (context == null || messageId == null) {
			throw new NullPointerException(" context "
					+ context
					+ " messageId "
					+ messageId);
		}
		Locale locale;
		// viewRoot may not have been initialized at this point.
		if (context.getViewRoot() != null) {
			locale = context.getViewRoot().getLocale();
		} else {
			locale = Locale.getDefault();
		}

		if (null == locale) {
			throw new NullPointerException(" locale is null ");
		}

		ResourceBundle bundle = ResourceBundle.getBundle(context.getApplication().getMessageBundle(), locale);

		String message = bundle.getString(messageId);
		if (params != null)
			return new FacesMessage(FacesMessage.SEVERITY_ERROR, MessageFormat.format(message, params), null);
		else
			return new FacesMessage(FacesMessage.SEVERITY_ERROR, MessageFormat.format(message, params), null);
	}

}
