package de.laliluna.howto.services.security;

import org.apache.log4j.Logger;
import org.slf4j.LoggerFactory;

import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginException;
import javax.security.auth.login.LoginContext;
import java.util.Properties;
import java.util.Map;
import java.util.Set;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * Taken from http://weblogs.java.net/blog/2006/03/07/repost-using-jaas-jsf
 */
public class JAASHelper {

	private Logger logger = Logger.getLogger(JAASHelper.class);

	private LoginContext loginContext;

	public boolean authenticate(String userid, String password) {
		boolean result = false;
		try {
			String loginName = "SampleLogin";

			loginContext = new LoginContext(loginName,
					new LoginCallback(userid, password));
			loginContext.login();
			result = true;
		}
		catch (LoginException e) {
			logger.error("Login failed", e);
			result = false;
		}
		return result;
	}

	public Subject getSubject() {
		Subject result = null;
		if (null != loginContext) {
			result = loginContext.getSubject();
		}
		return result;
	}

	public static class LoginCallback implements CallbackHandler {
		final org.slf4j.Logger logger = LoggerFactory.getLogger(LoginCallback.class);

		private String userName = null;
		private String password = null;

		public LoginCallback(String userName, String password) {
			this.userName = userName;
			this.password = password;
		}

		public void handle(Callback[] callbacks) {
			logger.info("Answering callback with user ({}) and password ({})", userName,password);
			for (int i = 0; i < callbacks.length; i++) {
				Logger.getLogger(LoginCallback.class).info(callbacks[i]);
				if (callbacks[i] instanceof NameCallback) {
					NameCallback nc = (NameCallback) callbacks[i];
					nc.setName(userName);
				} else if (callbacks[i] instanceof PasswordCallback && password != null) {
					PasswordCallback pc = (PasswordCallback) callbacks[i];

					pc.setPassword(password.toCharArray());
				}else{
					// work around as Jetty is using an ObjectCallback instead of a PasswordCallback
					if (System.getProperty("jetty.home") != null){
						Callback callback = callbacks[i];
						try {
							Method method = callback.getClass().getMethod("setObject", Object.class);
							method.invoke(callback,  password);
						} catch (NoSuchMethodException e) {
							throw new RuntimeException(e);
						} catch (InvocationTargetException e) {
							throw new RuntimeException(e);
						} catch (IllegalAccessException e) {
							throw new RuntimeException(e);
						}
					}

				}
			}
		}
	}

}
