package de.laliluna.howto.services.security;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.*;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

/**
 * JAAS Login Module. Can only be used as a single login module (to keep things simple). You might have a look at the provided modules of
 * the Jetty server to get an idea of a more flexible implementation.
 *
 * Developed from a sample provided by Rahul Bhattacharjee  on JavaRanch
 */
public class StaticLoginModule implements LoginModule {
	final Logger logger = LoggerFactory.getLogger(StaticLoginModule.class);

	private Subject subject;
	private CallbackHandler callbackHandler;
	//private Map state;
	//private Map options;

	private boolean commited;
	private String username;
	private List<String> roles;

	private static Map<String, String> userPassword = new HashMap<String, String>();

	static {
		userPassword.put("foo", "bar");
	}

	public boolean abort() throws LoginException {
		username = null;
		roles = Collections.EMPTY_LIST;
		return commited;
	}

	public boolean commit() throws LoginException {
			subject.getPrincipals().add(new JAASPrincipal(username));
			for (String role : roles) {
				subject.getPrincipals().add(new JAASRolePrincipal(role));
			}
		subject.getPrivateCredentials().add("bar");
			this.commited = true;
			return true;
	}

	public void initialize(Subject subject, CallbackHandler handler, Map state, Map options) {
		this.subject = subject;
		this.callbackHandler = handler;
		//this.state = state;
		//this.options = options;
	}

	public boolean login() throws LoginException {
		logger.debug("Login is starting");
		boolean returnValue = true;

		if (callbackHandler == null) {
			throw new LoginException("No callback handler supplied.");
		}

		Callback[] callbacks = new Callback[2];
		callbacks[0] = new NameCallback("Username");
		callbacks[1] = new PasswordCallback("Password", false);

		try {
			callbackHandler.handle(callbacks);
			String userName = ((NameCallback) callbacks[0]).getName();
			char[] passwordCharArray = ((PasswordCallback) callbacks[1]).getPassword();
			String password = passwordCharArray == null ? null : new String(passwordCharArray);

			logger.debug("User provided the following credentials, username={}, password={}", userName, password);
			if (userName != null) {
				String storedPassword = userPassword.get(userName);
				returnValue = (storedPassword != null) && storedPassword.equals(password);
				if (returnValue) {
					storeUserAndRoles(userName);
				}
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			throw new LoginException("IOException occured.");
		} catch (UnsupportedCallbackException e) {
			e.printStackTrace();
			throw new LoginException("UnsupportedCallbackException encountered.");
		}

		return returnValue;
	}

	/**
	 * We are not loading the roles from anywhere but just asume that every user has the role "foo"
	 *
	 * @param userName
	 */
	private void storeUserAndRoles(String userName) {
		this.username = userName;
		this.roles = Arrays.asList("foo");
	}

	public boolean logout() throws LoginException {
		subject.getPrincipals().remove(username);
		subject.getPrincipals().removeAll(roles);
		return true;
	}

}
