package de.laliluna.example.pages.wizard;

import de.laliluna.example.util.DialogState;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.SessionState;
import org.apache.tapestry5.PersistenceConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Base class for all wizard steps having access to the application state field article
 *
 * @author Sebastian Hennebrueder
 */
public abstract class AbstractWizard<T> {

	private Logger logger = LoggerFactory.getLogger(AbstractWizard.class);

	@SessionState()
	private DialogState<T> dialogState;

	private Integer dialogId;

	public Integer getDialogId() {
		if(dialogId == null)
			throw new IllegalStateException("Dialog id is null");
		return dialogId;
	}

	public T getModel() {
		return dialogState.getBean(dialogId);
	}

	public void startDialog(T bean) {
		dialogId = dialogState.startDialog(bean);
	}

	public Object continueDialog(AbstractWizard page) {
		logger.debug("Passing dialogId {} to page {}", dialogId, page);
		page.dialogId = dialogId;
		return page;
	}

	public void endDialog() {
		dialogState.endDialog(dialogId);
// destroy empty dialogState		
		if (dialogState.empty())
			dialogState = null;
	}

	Integer onPassivate(){
		return dialogId;
	}

	void onActivate(Integer dialogId) {
		this.dialogId = dialogId;
	}
}
