package de.laliluna.example.security;

import org.apache.tapestry5.ioc.MethodAdvice;
import org.apache.tapestry5.ioc.Invocation;

/**
 * User: hennebrueder
 * Date: 05.08.2009
 */
public class MethodSecurityAdvice implements MethodAdvice {

	private SecurityService securityService;

	private String rights[];

	/**
	 * @param securityService - security service to be used to load the user
	 * @param rightsAsString	- a comma separated list of right names, e.g. READ,READ_ALL, ADMIN
	 */
	public MethodSecurityAdvice(SecurityService securityService, String rightsAsString) {
		this.securityService = securityService;

		this.rights = rightsAsString.split(",");
		for (int i = 0; i < this.rights.length; i++) {
			this.rights[i] = this.rights[i].trim();
		}
	}

	public void advise(Invocation invocation) {
		ApplicationUser user = securityService.getUser();

		boolean hasRight = false;
		if (user != null) {
			for (String right : rights) {
				if (user.hasRight(right)) {
					hasRight = true;
					break;
				}
			}
		}
		if (hasRight)
			invocation.proceed();
		else
			throw new NotAuthorizedException("You are not allowed to access " + invocation.getMethodName());

	}

}
